"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var rxjs_1 = require('rxjs');
var slide_animator_1 = require('./slide.animator');
var slide_event_1 = require('./slide.event');
var SlideService = (function () {
    function SlideService() {
        var _this = this;
        this.slideChange = new rxjs_1.ReplaySubject();
        this.animating = false;
        this.config = {
            sensitivity: 20
        };
        this.keys = { 37: true, 38: true, 39: true, 40: true };
        this.keyObs = rxjs_1.Observable.fromEvent(document, 'keydown')
            .filter(function (event) { return _this.keys[event.keyCode]; });
        this.touchObs = rxjs_1.Observable.fromEvent(window, 'touchstart')
            .zip(rxjs_1.Observable.fromEvent(window, 'touchend'))
            .map(function (payload) {
            var anyPayload = payload;
            return {
                deltaY: anyPayload[0].changedTouches[0].clientY -
                    anyPayload[1].changedTouches[0].clientY
            };
        })
            .filter(function (change) { return Math.abs(change.deltaY) > 50; });
        this.scrollObs = rxjs_1.Observable.fromEvent(window, 'wheel')
            .merge(rxjs_1.Observable.fromEvent(window, 'mousewheel'))
            .merge(rxjs_1.Observable.fromEvent(window, 'DOMMouseScroll'))
            .merge(rxjs_1.Observable.fromEvent(window, 'touchmove'))
            .merge(this.touchObs)
            .merge(this.keyObs);
    }
    SlideService.prototype.init = function () {
        var _this = this;
        this.elements = this.getNg2SlideElements();
        this.animator = new slide_animator_1.SlideAnimator();
        this.scrollObs.subscribe(function (event) {
            if (window.innerHeight > 600) {
                _this.preventDefault(event);
                if (!_this.animating) {
                    _this.elements = _this.getNg2SlideElements();
                    var currentIndex = _this.getCurrentSlideIndex(_this.elements);
                    if (event.deltaY > _this.config.sensitivity || event.key === 'ArrowDown') {
                        _this.scrollToIndex(currentIndex + 1, currentIndex);
                    }
                    else if (event.deltaY < -_this.config.sensitivity || event.key === 'ArrowUp') {
                        _this.scrollToIndex(currentIndex - 1, currentIndex);
                    }
                }
            }
        });
    };
    SlideService.prototype.scrollToIndex = function (index, previousIndex, elements) {
        var _this = this;
        if (elements)
            this.elements = elements;
        if (!previousIndex)
            previousIndex = this.getCurrentSlideIndex(this.elements);
        this.slideChange.next(new slide_event_1.SlideEventStart(previousIndex, index));
        if (this.elements[index]) {
            var startingPoint = window.scrollY;
            var endingPoint = this.elements[index].getBoundingClientRect().top + window.scrollY;
            this.animating = true;
            this.animator.animateScroll(startingPoint, endingPoint, function () {
                _this.slideChange.next(new slide_event_1.SlideEventEnd(previousIndex, index));
                _this.animating = false;
            });
        }
    };
    SlideService.prototype.setConfiguration = function (config) {
        this.config = Object.assign(this.config, config);
    };
    SlideService.prototype.getNg2SlideElements = function () {
        return Array.prototype.slice.call(document.querySelectorAll('ng2-slide'));
    };
    SlideService.prototype.getCurrentSlideIndex = function (elements) {
        return elements.findIndex(function (element) {
            var offsetTop = element.getBoundingClientRect().top + window.scrollY;
            return offsetTop <= window.scrollY && window.scrollY < offsetTop + element.clientHeight;
        });
    };
    SlideService.prototype.preventDefault = function (e) {
        e = e || window.event;
        if (e.preventDefault)
            e.preventDefault();
        e.returnValue = false;
    };
    SlideService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [])
    ], SlideService);
    return SlideService;
}());
exports.SlideService = SlideService;
